@@ -6,6 +6,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <inttypes.h>
 
 #include "include/types.h"
 #include "include/utime.h"
@@ -301,6 +302,34 @@ int rgw_bucket_prepare_op(cls_method_context_t hctx, bufferlist *in, bufferlist
   return rc;
 }
 
+static void unaccount_entry(struct rgw_bucket_dir_header& header, struct rgw_bucket_dir_entry& entry)
+{
+  struct rgw_bucket_category_stats& stats = header.stats[entry.meta.category];
+  stats.num_entries--;
+  stats.total_size -= entry.meta.size;
+  stats.total_size_rounded -= get_rounded_size(entry.meta.size);
+}
+
+static int read_index_entry(cls_method_context_t hctx, string& name, struct rgw_bucket_dir_entry *entry)
+{
+  bufferlist current_entry;
+  int rc = cls_cxx_map_get_val(hctx, name, &current_entry);
+  if (rc < 0) {
+    return rc;
+  }
+
+  bufferlist::iterator cur_iter = current_entry.begin();
+  try {
+    ::decode(*entry, cur_iter);
+  } catch (buffer::error& err) {
+    CLS_LOG(1, "ERROR: read_index_entry(): failed to decode entry\n");
+    return -EIO;
+  }
+
+  CLS_LOG(1, "read_index_entry(): existing entry: epoch=%llu name=%s locator=%s\n", (unsigned long long)entry->epoch, entry->name.c_str(), entry->locator.c_str());
+  return 0;
+}
+
 int rgw_bucket_complete_op(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
 {
   // decode request
@@ -312,7 +341,7 @@ int rgw_bucket_complete_op(cls_method_context_t hctx, bufferlist *in, bufferlist
     CLS_LOG(1, "ERROR: rgw_bucket_complete_op(): failed to decode request\n");
     return -EINVAL;
   }
-  CLS_LOG(1, "rgw_bucket_complete_op(): request: op=%d name=%s epoch=%lld tag=%s\n", op.op, op.name.c_str(), op.epoch, op.tag.c_str());
+  CLS_LOG(1, "rgw_bucket_complete_op(): request: op=%d name=%s epoch=%llu tag=%s\n", op.op, op.name.c_str(), (unsigned long long)op.epoch, op.tag.c_str());
 
   bufferlist header_bl;
   struct rgw_bucket_dir_header header;
@@ -327,29 +356,18 @@ int rgw_bucket_complete_op(cls_method_context_t hctx, bufferlist *in, bufferlist
     return -EINVAL;
   }
 
-  bufferlist current_entry;
   struct rgw_bucket_dir_entry entry;
   bool ondisk = true;
-  rc = cls_cxx_map_get_val(hctx, op.name, &current_entry);
-  if (rc < 0) {
-    if (rc != -ENOENT) {
-      return rc;
-    } else {
-      rc = 0;
-      entry.name = op.name;
-      entry.epoch = op.epoch;
-      entry.meta = op.meta;
-      entry.locator = op.locator;
-      ondisk = false;
-    }
-  } else {
-    bufferlist::iterator cur_iter = current_entry.begin();
-    try {
-      ::decode(entry, cur_iter);
-      CLS_LOG(1, "rgw_bucket_complete_op(): existing entry: epoch=%lld name=%s locator=%s\n", entry.epoch, entry.name.c_str(), entry.locator.c_str());
-    } catch (buffer::error& err) {
-      CLS_LOG(1, "ERROR: rgw_bucket_complete_op(): failed to decode entry\n");
-    }
+
+  rc = read_index_entry(hctx, op.name, &entry);
+  if (rc == -ENOENT) {
+    entry.name = op.name;
+    entry.epoch = op.epoch;
+    entry.meta = op.meta;
+    entry.locator = op.locator;
+    ondisk = false;
+  } else if (rc < 0) {
+    return rc;
   }
 
   if (op.tag.size()) {
@@ -384,10 +402,7 @@ int rgw_bucket_complete_op(cls_method_context_t hctx, bufferlist *in, bufferlist
   }
 
   if (entry.exists) {
-    struct rgw_bucket_category_stats& stats = header.stats[entry.meta.category];
-    stats.num_entries--;
-    stats.total_size -= entry.meta.size;
-    stats.total_size_rounded -= get_rounded_size(entry.meta.size);
+    unaccount_entry(header, entry);
   }
 
   switch (op.op) {
@@ -429,6 +444,27 @@ int rgw_bucket_complete_op(cls_method_context_t hctx, bufferlist *in, bufferlist
     break;
   }
 
+  list<string>::iterator remove_iter;
+  CLS_LOG(0, "rgw_bucket_complete_op(): remove_objs.size()=%d\n", (int)op.remove_objs.size());
+  for (remove_iter = op.remove_objs.begin(); remove_iter != op.remove_objs.end(); ++remove_iter) {
+    string& remove_oid_name = *remove_iter;
+    CLS_LOG(1, "rgw_bucket_complete_op(): removing entries, read_index_entry name=%s\n", remove_oid_name.c_str());
+    struct rgw_bucket_dir_entry remove_entry;
+    int ret = read_index_entry(hctx, remove_oid_name, &remove_entry);
+    if (ret < 0) {
+      CLS_LOG(1, "rgw_bucket_complete_op(): removing entries, read_index_entry name=%s ret=%d\n", remove_oid_name.c_str(), rc);
+      continue;
+    }
+    CLS_LOG(0, "rgw_bucket_complete_op(): entry.name=%s entry.meta.category=%d\n", remove_entry.name.c_str(), remove_entry.meta.category);
+    unaccount_entry(header, remove_entry);
+
+    ret = cls_cxx_map_remove_key(hctx, remove_oid_name);
+    if (ret < 0) {
+      CLS_LOG(1, "rgw_bucket_complete_op(): cls_cxx_map_remove_key, failed to remove entry, name=%s read_index_entry ret=%d\n", remove_oid_name.c_str(), rc);
+      continue;
+    }
+  }
+
   bufferlist new_header_bl;
   ::encode(header, new_header_bl);
   return cls_cxx_map_write_header(hctx, &new_header_bl);
@@ -498,12 +534,12 @@ int rgw_dir_suggest_changes(cls_method_context_t hctx, bufferlist *in, bufferlis
 
     CLS_LOG(20, "cur_disk.pending_map.empty()=%d op=%d cur_disk.exists=%d cur_change.pending_map.size()=%d cur_change.exists=%d\n",
 	    cur_disk.pending_map.empty(), (int)op, cur_disk.exists,
-	    cur_change.pending_map.size(), cur_change.exists);
+	    (int)cur_change.pending_map.size(), cur_change.exists);
 
     if (cur_disk.pending_map.empty()) {
       if (cur_disk.exists) {
         struct rgw_bucket_category_stats& old_stats = header.stats[cur_disk.meta.category];
-        CLS_LOG(10, "total_entries: %d -> %d\n", old_stats.num_entries, old_stats.num_entries - 1);
+        CLS_LOG(10, "total_entries: %"PRId64" -> %"PRId64"\n", old_stats.num_entries, old_stats.num_entries - 1);
         old_stats.num_entries--;
         old_stats.total_size -= cur_disk.meta.size;
         old_stats.total_size_rounded -= get_rounded_size(cur_disk.meta.size);
@@ -519,7 +555,7 @@ int rgw_dir_suggest_changes(cls_method_context_t hctx, bufferlist *in, bufferlis
 	  return ret;
         break;
       case CEPH_RGW_UPDATE:
-        CLS_LOG(10, "CEPH_RGW_UPDATE name=%s total_entries: %d -> %d\n", cur_change.name.c_str(), stats.num_entries, stats.num_entries + 1);
+        CLS_LOG(10, "CEPH_RGW_UPDATE name=%s total_entries: %"PRId64" -> %"PRId64"\n", cur_change.name.c_str(), stats.num_entries, stats.num_entries + 1);
         stats.num_entries++;
         stats.total_size += cur_change.meta.size;
         stats.total_size_rounded += get_rounded_size(cur_change.meta.size);
@@ -710,7 +746,7 @@ static int usage_iterate_range(cls_method_context_t hctx, uint64_t start, uint64
         return 0;
       }
     }
-    iter--;
+    --iter;
     start_key = iter->first;
   } while (true);
   return 0;
@@ -866,7 +902,7 @@ static int gc_omap_remove(cls_method_context_t hctx, int type, const string& key
 static void get_time_key(utime_t& ut, string *key)
 {
   char buf[32];
-  snprintf(buf, 32, "%011lld.%09d", (long long)ut.sec(), ut.nsec());
+  snprintf(buf, 32, "%011llu.%09u", (unsigned long long)ut.sec(), ut.nsec());
   *key = buf;
 }
 
@@ -1035,7 +1071,7 @@ static int gc_iterate_entries(cls_method_context_t hctx, const string& marker,
       i++;
 
     }
-    iter--;
+    --iter;
     start_key = iter->first;
   } while (true);
   return 0;
