@@ -116,7 +116,7 @@ void crush_destroy(struct crush_map *map)
 	if (map->rules) {
 		__u32 b;
 		for (b = 0; b < map->max_rules; b++)
-			kfree(map->rules[b]);
+			crush_destroy_rule(map->rules[b]);
 		kfree(map->rules);
 	}
 
@@ -124,6 +124,11 @@ void crush_destroy(struct crush_map *map)
 	kfree(map);
 }
 
+void crush_destroy_rule(struct crush_rule *rule)
+{
+	kfree(rule);
+}
+
 // methods to check for safe arithmetic operations
 int crush_addition_is_unsafe(__u32 a, __u32 b)
 {
