@@ -125,6 +125,10 @@ void global_init(std::vector < const char * > *alt_def_args, std::vector < const
   }
   register_assert_context(cct);
 
+  // call all observers now.  this has the side-effect of configuring
+  // and opening the log file immediately.
+  conf->call_all_observers();
+
   if (code_env == CODE_ENVIRONMENT_DAEMON)
     output_ceph_version();
 }
@@ -139,16 +143,24 @@ static void pidfile_remove_void(void)
   pidfile_remove();
 }
 
-void global_init_daemonize(CephContext *cct, int flags)
+int global_init_prefork(CephContext *cct, int flags)
 {
   if (g_code_env != CODE_ENVIRONMENT_DAEMON)
-    return;
+    return -1;
   const md_config_t *conf = cct->_conf;
   if (!conf->daemonize)
-    return;
+    return -1;
 
   // stop log thread
+  g_ceph_context->_log->flush();
   g_ceph_context->_log->stop();
+  return 0;
+}
+
+void global_init_daemonize(CephContext *cct, int flags)
+{
+  if (global_init_prefork(cct, flags) < 0)
+    return;
 
   int ret = daemon(1, 1);
   if (ret) {
@@ -158,6 +170,13 @@ void global_init_daemonize(CephContext *cct, int flags)
     exit(1);
   }
 
+  global_init_postfork(cct, flags);
+}
+
+void global_init_postfork(CephContext *cct, int flags)
+{
+  int ret;
+
   // restart log thread
   g_ceph_context->_log->start();
 
