@@ -644,7 +644,7 @@ void CDir::remove_null_dentries() {
   CDir::map_t::iterator p = items.begin();
   while (p != items.end()) {
     CDentry *dn = p->second;
-    p++;
+    ++p;
     if (dn->get_linkage()->is_null() && !dn->is_projected())
       remove_dentry(dn);
   }
@@ -687,7 +687,7 @@ void CDir::purge_stale_snap_data(const set<snapid_t>& snaps)
   CDir::map_t::iterator p = items.begin();
   while (p != items.end()) {
     CDentry *dn = p->second;
-    p++;
+    ++p;
 
     if (dn->last == CEPH_NOSNAP)
       continue;
@@ -931,7 +931,7 @@ void CDir::merge(list<CDir*>& subs, list<Context*>& waiters, bool replay)
   int stale_rstat = 0;
   int stale_fragstat = 0;
 
-  for (list<CDir*>::iterator p = subs.begin(); p != subs.end(); p++) {
+  for (list<CDir*>::iterator p = subs.begin(); p != subs.end(); ++p) {
     CDir *dir = *p;
     dout(10) << " subfrag " << dir->get_frag() << " " << *dir << dendl;
     assert(!dir->is_auth() || dir->is_complete() || replay);
@@ -1022,6 +1022,7 @@ void CDir::assimilate_dirty_rstat_inodes()
   for (elist<CInode*>::iterator p = dirty_rstat_inodes.begin_use_current();
        !p.end(); ++p) {
     CInode *in = *p;
+    assert(in->is_auth());
     if (in->is_frozen())
       continue;
 
@@ -1553,16 +1554,59 @@ void CDir::_fetched(bufferlist &bl, const string& want_dn)
       if (stale)
 	continue;
 
+      bool undef_inode = false;
       if (dn) {
-        if (dn->get_linkage()->get_inode() == 0) {
-          dout(12) << "_fetched  had NEG dentry " << *dn << dendl;
-        } else {
-          dout(12) << "_fetched  had dentry " << *dn << dendl;
-        }
-      } else {
+	CInode *in = dn->get_linkage()->get_inode();
+	if (in) {
+	  dout(12) << "_fetched  had dentry " << *dn << dendl;
+	  if (in->state_test(CInode::STATE_REJOINUNDEF)) {
+	    assert(cache->mds->is_rejoin());
+	    assert(in->vino() == vinodeno_t(inode.ino, last));
+	    in->state_clear(CInode::STATE_REJOINUNDEF);
+	    cache->opened_undef_inode(in);
+	    undef_inode = true;
+	  }
+	} else
+	  dout(12) << "_fetched  had NEG dentry " << *dn << dendl;
+      }
+
+      if (!dn || undef_inode) {
 	// add inode
 	CInode *in = cache->get_inode(inode.ino, last);
-	if (in) {
+	if (!in || undef_inode) {
+	  if (undef_inode)
+	    in->first = first;
+	  else
+	    in = new CInode(cache, true, first, last);
+	  
+	  in->inode = inode;
+	  // symlink?
+	  if (in->is_symlink()) 
+	    in->symlink = symlink;
+	  
+	  in->dirfragtree.swap(fragtree);
+	  in->xattrs.swap(xattrs);
+	  in->decode_snap_blob(snapbl);
+	  in->old_inodes.swap(old_inodes);
+	  if (snaps)
+	    in->purge_stale_snap_data(*snaps);
+
+	  if (undef_inode) {
+	    if (inode.anchored)
+	      dn->adjust_nested_anchors(1);
+	  } else {
+	    cache->add_inode( in ); // add
+	    dn = add_primary_dentry(dname, in, first, last); // link
+	  }
+	  dout(12) << "_fetched  got " << *dn << " " << *in << dendl;
+
+	  if (in->inode.is_dirty_rstat())
+	    in->mark_dirty_rstat();
+
+	  //in->hack_accessed = false;
+	  //in->hack_load_stamp = ceph_clock_now(g_ceph_context);
+	  //num_new_inodes_loaded++;
+	} else {
 	  dout(0) << "_fetched  badness: got (but i already had) " << *in
 		  << " mode " << in->inode.mode
 		  << " mtime " << in->inode.mtime << dendl;
@@ -1575,35 +1619,6 @@ void CDir::_fetched(bufferlist &bl, const string& want_dn)
 	    << ", but inode " << in->vino() << " v" << in->inode.version
 	    << " already exists at " << inopath << "\n";
 	  continue;
-	} else {
-	  // inode
-	  in = new CInode(cache, true, first, last);
-	  in->inode = inode;
-	  
-	  // symlink?
-	  if (in->is_symlink()) 
-	    in->symlink = symlink;
-	  
-	  in->dirfragtree.swap(fragtree);
-	  in->xattrs.swap(xattrs);
-	  in->decode_snap_blob(snapbl);
-	  in->old_inodes.swap(old_inodes);
-	  if (snaps)
-	    in->purge_stale_snap_data(*snaps);
-
-	  // add 
-	  cache->add_inode( in );
-	
-	  // link
-	  dn = add_primary_dentry(dname, in, first, last);
-	  dout(12) << "_fetched  got " << *dn << " " << *in << dendl;
-
-	  if (in->inode.is_dirty_rstat())
-	    in->mark_dirty_rstat();
-
-	  //in->hack_accessed = false;
-	  //in->hack_load_stamp = ceph_clock_now(g_ceph_context);
-	  //num_new_inodes_loaded++;
 	}
       }
     } else {
@@ -1744,7 +1759,7 @@ CDir::map_t::iterator CDir::_commit_full(ObjectOperation& m, const set<snapid_t>
   map_t::iterator p = items.begin();
   while (p != items.end() && bl.length() < max_write_size) {
     CDentry *dn = p->second;
-    p++;
+    ++p;
     
     if (dn->linkage.is_null()) 
       continue;  // skip negative entries
@@ -1962,12 +1977,6 @@ void CDir::_commit(version_t want)
   map_t::iterator committed_dn;
   unsigned max_write_size = cache->max_dir_commit_size;
 
-  // update parent pointer while we're here.
-  //  NOTE: the pointer is ONLY required to be valid for the first frag.  we put the xattr
-  //        on other frags too because it can't hurt, but it won't necessarily be up to date
-  //        in that case!!
-  max_write_size -= inode->encode_parent_mutation(m);
-
   if (is_complete() &&
       (num_dirty > (num_head_items*g_conf->mds_dir_commit_ratio))) {
     fnode.snap_purged_thru = realm->get_last_destroyed();
@@ -2025,16 +2034,6 @@ void CDir::_committed(version_t v, version_t lrv)
 
   bool stray = inode->is_stray();
 
-  // did we update the parent pointer too?
-  if (get_frag() == frag_t() &&     // only counts on first frag
-      inode->state_test(CInode::STATE_DIRTYPARENT) &&
-      lrv == inode->inode.last_renamed_version) {
-    inode->item_renamed_file.remove_myself();
-    inode->state_clear(CInode::STATE_DIRTYPARENT);
-    inode->put(CInode::PIN_DIRTYPARENT);
-    dout(10) << "_committed  stored parent pointer, removed from renamed_files list " << *inode << dendl;
-  }
-  
   // take note.
   assert(v > committed_version);
   assert(v <= committing_version);
@@ -2055,7 +2054,7 @@ void CDir::_committed(version_t v, version_t lrv)
   for (map_t::iterator it = items.begin();
        it != items.end(); ) {
     CDentry *dn = it->second;
-    it++;
+    ++it;
     
     // inode?
     if (dn->linkage.is_primary()) {
@@ -2098,7 +2097,7 @@ void CDir::_committed(version_t v, version_t lrv)
   map<version_t, list<Context*> >::iterator p = waiting_for_commit.begin();
   while (p != waiting_for_commit.end()) {
     map<version_t, list<Context*> >::iterator n = p;
-    n++;
+    ++n;
     if (p->first > committed_version) {
       dout(10) << " there are waiters for " << p->first << ", committing again" << dendl;
       _commit(p->first);
@@ -2150,7 +2149,7 @@ void CDir::finish_export(utime_t now)
   dirty_old_rstat.clear();
 }
 
-void CDir::decode_import(bufferlist::iterator& blp, utime_t now)
+void CDir::decode_import(bufferlist::iterator& blp, utime_t now, LogSegment *ls)
 {
   ::decode(first, blp);
   ::decode(fnode, blp);
@@ -2163,7 +2162,10 @@ void CDir::decode_import(bufferlist::iterator& blp, utime_t now)
   ::decode(s, blp);
   state &= MASK_STATE_IMPORT_KEPT;
   state |= (s & MASK_STATE_EXPORTED);
-  if (is_dirty()) get(PIN_DIRTY);
+  if (is_dirty()) {
+    get(PIN_DIRTY);
+    _mark_dirty(ls);
+  }
 
   ::decode(dir_rep, blp);
 
@@ -2400,7 +2402,7 @@ void CDir::verify_fragstat()
 
   for (map_t::iterator it = items.begin();
        it != items.end();
-       it++) {
+       ++it) {
     CDentry *dn = it->second;
     if (dn->is_null())
       continue;
