@@ -222,7 +222,8 @@ bool MonmapMonitor::preprocess_command(MMonCommand *m)
       for (std::vector<const char*>::iterator i = args.begin()+1; i != args.end(); ) {
 	if (ceph_argparse_double_dash(args, i))
 	  break;
-	else if (ceph_argparse_witharg(args, i, &val, "-f", "--format", (char*)NULL))
+	else if (ceph_argparse_witharg_daemon(args, i, &val, "-f", "--format",
+					      (char*)NULL))
 	  format = val;
 	else if (!epoch) {
 	  long l = parse_pos_long(*i++, &ss);
@@ -232,7 +233,7 @@ bool MonmapMonitor::preprocess_command(MMonCommand *m)
 	  }
 	  epoch = l;
 	} else
-	  i++;
+	  ++i;
       }
 
       MonMap *p = mon->monmap;
@@ -404,6 +405,11 @@ bool MonmapMonitor::prepare_command(MMonCommand *m)
         goto out;
       }
 
+      if (pending_map.size() == 1) {
+	err = -EINVAL;
+	ss << "error: refusing removal of last monitor " << name;
+	goto out;
+      }
       entity_addr_t addr = pending_map.get_addr(name);
       pending_map.remove(name);
       pending_map.last_changed = ceph_clock_now(g_ceph_context);
@@ -496,3 +502,33 @@ void MonmapMonitor::get_health(list<pair<health_status_t, string> >& summary,
     }
   }
 }
+
+int MonmapMonitor::get_monmap(bufferlist &bl)
+{
+  version_t latest_ver = get_last_committed();
+  dout(10) << __func__ << " ver " << latest_ver << dendl;
+
+  if (!exists_version(latest_ver))
+    return -ENOENT;
+
+  int err = get_version(latest_ver, bl);
+  if (err < 0) {
+    dout(1) << __func__ << " error obtaining monmap: "
+            << cpp_strerror(err) << dendl;
+    return err;
+  }
+  return 0;
+}
+
+int MonmapMonitor::get_monmap(MonMap &m)
+{
+  dout(10) << __func__ << dendl;
+  bufferlist monmap_bl;
+
+  int err = get_monmap(monmap_bl);
+  if (err < 0) {
+    return err;
+  }
+  m.decode(monmap_bl);
+  return 0;
+}
