@@ -30,6 +30,7 @@ using namespace std;
 #include "include/utime.h"
 #include "msg/Messenger.h"
 #include "common/config.h"
+#include "mon/MonitorDBStore.h"
 
 #include "messages/MPGStats.h"
 #include "messages/MPGStatsAck.h"
@@ -38,6 +39,7 @@ class MMonCommand;
 class MGetPoolStats;
 
 class RatioMonitor;
+class TextTable;
 
 class PGMonitor : public PaxosService {
 public:
@@ -50,10 +52,13 @@ private:
 
   void create_initial();
   void update_from_paxos();
+  void init();
   void handle_osd_timeouts();
   void create_pending();  // prepare a new pending
-  void encode_pending(bufferlist &bl);  // propose pending update to peers
-
+  // propose pending update to peers
+  void update_trim();
+  void encode_pending(MonitorDBStore::Transaction *t);
+  virtual void encode_full(MonitorDBStore::Transaction *t);
   void update_logger();
 
   bool preprocess_query(PaxosServiceMessage *m);  // true if processed.
@@ -105,6 +110,7 @@ private:
    */
   bool register_new_pgs();
 
+  void map_pg_creates();
   void send_pg_creates();
   void send_pg_creates(int osd, Connection *con);
 
@@ -126,9 +132,13 @@ private:
 			  bufferlist& rdata,
 			  vector<const char*>& args) const;
 
+  void dump_object_stat_sum(TextTable &tbl, Formatter *f,
+                            object_stat_sum_t &sum, bool verbose);
+
 public:
-  PGMonitor(Monitor *mn, Paxos *p);
-  virtual ~PGMonitor();
+  PGMonitor(Monitor *mn, Paxos *p, const string& service_name)
+  : PaxosService(mn, p, service_name), need_check_down_pgs(false) { }
+  ~PGMonitor() { }
 
   virtual void on_restart();
 
@@ -141,6 +151,9 @@ public:
 
   void check_osd_map(epoch_t epoch);
 
+  void dump_pool_stats(stringstream &ss, Formatter *f, bool verbose);
+  void dump_fs_stats(stringstream &ss, Formatter *f, bool verbose);
+
   void dump_info(Formatter *f);
 
   void get_health(list<pair<health_status_t,string> >& summary,
