@@ -58,6 +58,7 @@ using namespace std;
 #include "messages/MOSDAlive.h"
 #include "messages/MOSDPGTemp.h"
 #include "messages/MOSDFailure.h"
+#include "messages/MOSDMarkMeDown.h"
 #include "messages/MOSDPing.h"
 #include "messages/MOSDOp.h"
 #include "messages/MOSDOpReply.h"
@@ -371,6 +372,9 @@ Message *decode_message(CephContext *cct, ceph_msg_header& header, ceph_msg_foot
   case MSG_OSD_FAILURE:
     m = new MOSDFailure();
     break;
+  case MSG_OSD_MARK_ME_DOWN:
+    m = new MOSDMarkMeDown();
+    break;
   case MSG_OSD_PING:
     m = new MOSDPing();
     break;
