@@ -74,7 +74,8 @@ public:
      *  the associated Connection(s). When reading in a new Message, the Messenger
      *  will call throttler->throttle() for the size of the new Message.
      */
-    Throttle *throttler;
+    Throttle *throttler_bytes;
+    Throttle *throttler_messages;
 
     /// Specify features supported locally by the endpoint.
     uint64_t features_supported;
@@ -82,12 +83,16 @@ public:
     uint64_t features_required;
 
     Policy()
-      : lossy(false), server(false), standby(false), resetcheck(true), throttler(NULL),
+      : lossy(false), server(false), standby(false), resetcheck(true),
+	throttler_bytes(NULL),
+	throttler_messages(NULL),
 	features_supported(CEPH_FEATURES_SUPPORTED_DEFAULT),
 	features_required(0) {}
   private:
     Policy(bool l, bool s, bool st, bool r, uint64_t sup, uint64_t req)
-      : lossy(l), server(s), standby(st), resetcheck(r), throttler(NULL),
+      : lossy(l), server(s), standby(st), resetcheck(r),
+	throttler_bytes(NULL),
+	throttler_messages(NULL),
 	features_supported(sup | CEPH_FEATURES_SUPPORTED_DEFAULT),
 	features_required(req) {}
 
@@ -184,7 +189,7 @@ public:
    *
    * @param m The name to set.
    */
-  void set_myname(const entity_name_t m) { my_inst.name = m; }
+  void set_myname(const entity_name_t& m) { my_inst.name = m; }
   /**
    * Set the unknown address components for this Messenger.
    * This is useful if the Messenger doesn't know its full address just by
@@ -202,6 +207,13 @@ public:
    * but not yet dispatched.
    */
   virtual int get_dispatch_queue_len() = 0;
+
+  /**
+   * Get age of oldest undelivered message
+   * (0 if the queue is empty)
+   */
+  virtual double get_dispatch_queue_max_age(utime_t now) = 0;
+
   /**
    * @} // Accessors
    */
@@ -266,7 +278,7 @@ public:
    * ownership of this pointer, but you must not destroy it before
    * you destroy the Messenger.
    */
-  virtual void set_policy_throttler(int type, Throttle *t) = 0;
+  virtual void set_policy_throttlers(int type, Throttle *bytes, Throttle *msgs=NULL) = 0;
   /**
    * Set the default send priority
    *
@@ -476,6 +488,7 @@ public:
    * discard its outgoing queue, and if the endpoint tries
    * to reconnect they will discard their queue when we
    * inform them of the session reset.
+   * If the Connection* is NULL, this is a no-op.
    * It does not generate any notifications to the Dispatcher.
    *
    * @param con The Connection to mark down.
@@ -549,14 +562,14 @@ public:
     m->set_dispatch_stamp(ceph_clock_now(cct));
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p) {
       if ((*p)->ms_dispatch(m))
 	return;
-    std::ostringstream oss;
-    oss << "ms_deliver_dispatch: fatal error: unhandled message "
-	<< m << " " << *m << " from " << m->get_source_inst();
-    dout_emergency(oss.str());
-    assert(0);
+    }
+    lsubdout(cct, ms, 0) << "ms_deliver_dispatch: unhandled message " << m << " " << *m << " from "
+			 << m->get_source_inst() << dendl;
+    assert(!cct->_conf->ms_die_on_unhandled_msg);
+    m->put();
   }
   /**
    * Notify each Dispatcher of a new Connection. Call
@@ -567,7 +580,7 @@ public:
   void ms_deliver_handle_connect(Connection *con) {
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p)
       (*p)->ms_handle_connect(con);
   }
 
@@ -580,7 +593,7 @@ public:
   void ms_deliver_handle_accept(Connection *con) {
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p)
       (*p)->ms_handle_accept(con);
   }
 
@@ -594,9 +607,10 @@ public:
   void ms_deliver_handle_reset(Connection *con) {
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p) {
       if ((*p)->ms_handle_reset(con))
 	return;
+    }
   }
   /**
    * Notify each Dispatcher of a Connection which has been "forgotten" about
@@ -608,7 +622,7 @@ public:
   void ms_deliver_handle_remote_reset(Connection *con) {
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p)
       (*p)->ms_handle_remote_reset(con);
   }
   /**
@@ -622,9 +636,10 @@ public:
     AuthAuthorizer *a = 0;
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p) {
       if ((*p)->ms_get_authorizer(peer_type, &a, force_new))
 	return a;
+    }
     return NULL;
   }
   /**
@@ -646,11 +661,13 @@ public:
 				    bool& isvalid, CryptoKey& session_key) {
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
-	 p++)
+	 ++p) {
       if ((*p)->ms_verify_authorizer(con, peer_type, protocol, authorizer, authorizer_reply, isvalid, session_key))
 	return true;
+    }
     return false;
   }
+
   /**
    * @} // Dispatcher Interfacing
    */
