@@ -110,6 +110,13 @@ public:
   int get_dispatch_queue_len() {
     return dispatch_queue.get_queue_len();
   }
+  /**
+   * Get age of oldest undelivered message
+   * (0 if the queue is empty)
+   */
+  double get_dispatch_queue_max_age(utime_t now) {
+    return dispatch_queue.get_max_age(now);
+  }
   /** @} Accessors */
 
   /**
@@ -162,12 +169,15 @@ public:
    * ownership of this pointer, but you must not destroy it before
    * you destroy SimpleMessenger.
    */
-  void set_policy_throttler(int type, Throttle *t) {
+  void set_policy_throttlers(int type, Throttle *byte_throttle, Throttle *msg_throttle) {
     Mutex::Locker l(policy_lock);
-    if (policy_map.count(type))
-      policy_map[type].throttler = t;
-    else
-      default_policy.throttler = t;
+    if (policy_map.count(type)) {
+      policy_map[type].throttler_bytes = byte_throttle;
+      policy_map[type].throttler_messages = msg_throttle;
+    } else {
+      default_policy.throttler_bytes = byte_throttle;
+      default_policy.throttler_messages = msg_throttle;
+    }
   }
   /**
    * Bind the SimpleMessenger to a specific address. If bind_addr
