@@ -152,6 +152,9 @@ public:
       OP_OMAP_SETKEYS = 32, // cid, attrset
       OP_OMAP_RMKEYS = 33,  // cid, keyset
       OP_OMAP_SETHEADER = 34, // cid, header
+      OP_SPLIT_COLLECTION = 35, // cid, bits, destination
+      OP_SPLIT_COLLECTION2 = 36, /* cid, bits, destination
+				    doesn't create the destination */
     };
 
   private:
@@ -162,17 +165,74 @@ public:
     bool sobject_encoding;
     int64_t pool_override;
     bool use_pool_override;
+    bool replica;
+    bool tolerate_collection_add_enoent;
+
+    list<Context *> on_applied;
+    list<Context *> on_commit;
+    list<Context *> on_applied_sync;
 
   public:
+    void set_tolerate_collection_add_enoent() {
+      tolerate_collection_add_enoent = true;
+    }
+    void register_on_applied(Context *c) {
+      on_applied.push_back(c);
+    }
+    void register_on_commit(Context *c) {
+      on_commit.push_back(c);
+    }
+    void register_on_applied_sync(Context *c) {
+      on_applied_sync.push_back(c);
+    }
+
+    static void collect_contexts(
+      list<Transaction *> &t,
+      Context **out_on_applied,
+      Context **out_on_commit,
+      Context **out_on_applied_sync) {
+      assert(out_on_applied);
+      assert(out_on_commit);
+      assert(out_on_applied_sync);
+      list<Context *> on_applied, on_commit, on_applied_sync;
+      for (list<Transaction *>::iterator i = t.begin();
+	   i != t.end();
+	   ++i) {
+	on_applied.splice(on_applied.end(), (*i)->on_applied);
+	on_commit.splice(on_commit.end(), (*i)->on_commit);
+	on_applied_sync.splice(on_applied_sync.end(), (*i)->on_applied_sync);
+      }
+      *out_on_applied = C_Contexts::list_to_context(on_applied);
+      *out_on_commit = C_Contexts::list_to_context(on_commit);
+      *out_on_applied_sync = C_Contexts::list_to_context(on_applied_sync);
+    }
+
+    Context *get_on_applied() {
+      return C_Contexts::list_to_context(on_applied);
+    }
+    Context *get_on_commit() {
+      return C_Contexts::list_to_context(on_commit);
+    }
+    Context *get_on_applied_sync() {
+      return C_Contexts::list_to_context(on_applied_sync);
+    }
+
     void set_pool_override(int64_t pool) {
       pool_override = pool;
     }
+    void set_replica() {
+      replica = true;
+    }
+    bool get_replica() { return replica; }
 
     void swap(Transaction& other) {
       std::swap(ops, other.ops);
       std::swap(largest_data_len, other.largest_data_len);
       std::swap(largest_data_off, other.largest_data_off);
       std::swap(largest_data_off_in_tbl, other.largest_data_off_in_tbl);
+      std::swap(on_applied, other.on_applied);
+      std::swap(on_commit, other.on_commit);
+      std::swap(on_applied_sync, other.on_applied_sync);
       tbl.swap(other.tbl);
     }
 
@@ -186,6 +246,9 @@ public:
 	largest_data_off_in_tbl = tbl.length() + other.largest_data_off_in_tbl;
       }
       tbl.append(other.tbl);
+      on_applied.splice(on_applied.end(), other.on_applied);
+      on_commit.splice(on_commit.end(), other.on_commit);
+      on_applied_sync.splice(on_applied_sync.end(), other.on_applied_sync);
     }
 
     uint64_t get_encoded_bytes() {
@@ -234,16 +297,24 @@ public:
       bool sobject_encoding;
       int64_t pool_override;
       bool use_pool_override;
+      bool replica;
+      bool _tolerate_collection_add_enoent;
 
       iterator(Transaction *t)
 	: p(t->tbl.begin()),
 	  sobject_encoding(t->sobject_encoding),
 	  pool_override(t->pool_override),
-	  use_pool_override(t->use_pool_override) {}
+	  use_pool_override(t->use_pool_override),
+	  replica(t->replica),
+	  _tolerate_collection_add_enoent(
+	    t->tolerate_collection_add_enoent) {}
 
       friend class Transaction;
 
     public:
+      bool tolerate_collection_add_enoent() const {
+	return _tolerate_collection_add_enoent;
+      }
       bool have_op() {
 	return !p.end();
       }
@@ -295,6 +366,12 @@ public:
       void get_keyset(set<string> &keys) {
 	::decode(keys, p);
       }
+      uint32_t get_u32() {
+	uint32_t bits;
+	::decode(bits, p);
+	return bits;
+      }
+      bool get_replica() { return replica; }
     };
 
     iterator begin() {
@@ -543,34 +620,58 @@ public:
       ops++;
     }
 
+    /// Split collection based on given prefixes
+    void split_collection(
+      coll_t cid,
+      uint32_t bits,
+      uint32_t rem,
+      coll_t destination) {
+      __u32 op = OP_SPLIT_COLLECTION2;
+      ::encode(op, tbl);
+      ::encode(cid, tbl);
+      ::encode(bits, tbl);
+      ::encode(rem, tbl);
+      ::encode(destination, tbl);
+      ++ops;
+    }
+
     // etc.
     Transaction() :
       ops(0), pad_unused_bytes(0), largest_data_len(0), largest_data_off(0), largest_data_off_in_tbl(0),
-      sobject_encoding(false), pool_override(-1), use_pool_override(false) {}
+      sobject_encoding(false), pool_override(-1), use_pool_override(false),
+      replica(false),
+      tolerate_collection_add_enoent(false) {}
+
     Transaction(bufferlist::iterator &dp) :
       ops(0), pad_unused_bytes(0), largest_data_len(0), largest_data_off(0), largest_data_off_in_tbl(0),
-      sobject_encoding(false), pool_override(-1), use_pool_override(false) {
+      sobject_encoding(false), pool_override(-1), use_pool_override(false),
+      replica(false),
+      tolerate_collection_add_enoent(false) {
       decode(dp);
     }
+
     Transaction(bufferlist &nbl) :
       ops(0), pad_unused_bytes(0), largest_data_len(0), largest_data_off(0), largest_data_off_in_tbl(0),
-      sobject_encoding(false), pool_override(-1), use_pool_override(false) {
+      sobject_encoding(false), pool_override(-1), use_pool_override(false),
+      replica(false),
+      tolerate_collection_add_enoent(false) {
       bufferlist::iterator dp = nbl.begin();
       decode(dp); 
     }
 
     void encode(bufferlist& bl) const {
-      ENCODE_START(6, 5, bl);
+      ENCODE_START(7, 5, bl);
       ::encode(ops, bl);
       ::encode(pad_unused_bytes, bl);
       ::encode(largest_data_len, bl);
       ::encode(largest_data_off, bl);
       ::encode(largest_data_off_in_tbl, bl);
       ::encode(tbl, bl);
+      ::encode(tolerate_collection_add_enoent, bl);
       ENCODE_FINISH(bl);
     }
     void decode(bufferlist::iterator &bl) {
-      DECODE_START_LEGACY_COMPAT_LEN(6, 5, 5, bl);
+      DECODE_START_LEGACY_COMPAT_LEN(7, 5, 5, bl);
       DECODE_OLDEST(2);
       if (struct_v < 4)
 	sobject_encoding = true;
@@ -584,10 +685,13 @@ public:
 	::decode(largest_data_off_in_tbl, bl);
       }
       ::decode(tbl, bl);
-      DECODE_FINISH(bl);
       if (struct_v < 6) {
 	use_pool_override = true;
       }
+      if (struct_v >= 7) {
+	::decode(tolerate_collection_add_enoent, bl);
+      }
+      DECODE_FINISH(bl);
     }
 
     void dump(ceph::Formatter *f);
@@ -612,20 +716,74 @@ public:
     }
   };
 
-  virtual unsigned apply_transaction(Transaction& t, Context *ondisk=0) = 0;
-  virtual unsigned apply_transactions(list<Transaction*>& tls, Context *ondisk=0) = 0;
+  // synchronous wrappers
+  unsigned apply_transaction(Transaction& t, Context *ondisk=0) {
+    list<Transaction*> tls;
+    tls.push_back(&t);
+    return apply_transactions(NULL, tls, ondisk);
+  }
+  unsigned apply_transaction(Sequencer *osr, Transaction& t, Context *ondisk=0) {
+    list<Transaction*> tls;
+    tls.push_back(&t);
+    return apply_transactions(osr, tls, ondisk);
+  }
+  unsigned apply_transactions(list<Transaction*>& tls, Context *ondisk=0) {
+    return apply_transactions(NULL, tls, ondisk);
+  }
+  unsigned apply_transactions(Sequencer *osr, list<Transaction*>& tls, Context *ondisk=0);
 
-  virtual int queue_transaction(Sequencer *osr, Transaction* t) = 0;
-  virtual int queue_transaction(Sequencer *osr, Transaction *t, Context *onreadable, Context *ondisk=0,
+  int queue_transaction(Sequencer *osr, Transaction* t) {
+    list<Transaction *> tls;
+    tls.push_back(t);
+    return queue_transactions(osr, tls, new C_DeleteTransaction(t));
+  }
+
+  int queue_transaction(Sequencer *osr, Transaction *t, Context *onreadable, Context *ondisk=0,
 				Context *onreadable_sync=0,
 				TrackedOpRef op = TrackedOpRef()) {
     list<Transaction*> tls;
     tls.push_back(t);
     return queue_transactions(osr, tls, onreadable, ondisk, onreadable_sync, op);
   }
-  virtual int queue_transactions(Sequencer *osr, list<Transaction*>& tls, Context *onreadable, Context *ondisk=0,
-				 Context *onreadable_sync=0,
-				 TrackedOpRef op = TrackedOpRef()) = 0;
+
+  int queue_transactions(Sequencer *osr, list<Transaction*>& tls,
+			 Context *onreadable, Context *ondisk=0,
+			 Context *onreadable_sync=0,
+			 TrackedOpRef op = TrackedOpRef()) {
+    assert(!tls.empty());
+    tls.back()->register_on_applied(onreadable);
+    tls.back()->register_on_commit(ondisk);
+    tls.back()->register_on_applied_sync(onreadable_sync);
+    return queue_transactions(osr, tls, op);
+  }
+
+  virtual int queue_transactions(
+    Sequencer *osr, list<Transaction*>& tls,
+    TrackedOpRef op = TrackedOpRef()) = 0;
+
+
+  int queue_transactions(
+    Sequencer *osr,
+    list<Transaction*>& tls,
+    Context *onreadable,
+    Context *oncommit,
+    Context *onreadable_sync,
+    Context *oncomplete,
+    TrackedOpRef op);
+
+  int queue_transaction(
+    Sequencer *osr,
+    Transaction* t,
+    Context *onreadable,
+    Context *oncommit,
+    Context *onreadable_sync,
+    Context *oncomplete,
+    TrackedOpRef op) {
+    list<Transaction*> tls;
+    tls.push_back(t);
+    return queue_transactions(
+      osr, tls, onreadable, oncommit, onreadable_sync, oncomplete, op);
+  }
 
  public:
   ObjectStore() : logger(NULL) {}
@@ -659,8 +817,20 @@ public:
 
   // objects
   virtual bool exists(coll_t cid, const hobject_t& oid) = 0;                   // useful?
-  virtual int stat(coll_t cid, const hobject_t& oid, struct stat *st) = 0;     // struct stat?
-  virtual int read(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len, bufferlist& bl) = 0;
+  virtual int stat(
+    coll_t cid,
+    const hobject_t& oid,
+    struct stat *st,
+    bool allow_eio = false) = 0; // struct stat?
+
+  virtual int read(
+    coll_t cid,
+    const hobject_t& oid,
+    uint64_t offset,
+    size_t len,
+    bufferlist& bl,
+    bool allow_eio = false) = 0;
+
   virtual int fiemap(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len, bufferlist& bl) = 0;
 
   virtual int getattr(coll_t cid, const hobject_t& oid, const char *name, bufferptr& value) = 0;
@@ -704,6 +874,19 @@ public:
 				      int min, int max, snapid_t snap, 
 				      vector<hobject_t> *ls, hobject_t *next) = 0;
 
+  /**
+   * list contents of a collection that fall in the range [start, end)
+   *
+   * @param c collection
+   * @param start list object that sort >= this value
+   * @param end list objects that sort < this value
+   * @param snapid return no objects with snap < snapid
+   * @param ls [out] result
+   * @return zero on success, or negative error
+   */
+  virtual int collection_list_range(coll_t c, hobject_t start, hobject_t end,
+                                    snapid_t seq, vector<hobject_t> *ls) = 0;
+
   /// OMAP
   /// Get omap contents
   virtual int omap_get(
@@ -717,7 +900,8 @@ public:
   virtual int omap_get_header(
     coll_t c,                ///< [in] Collection containing hoid
     const hobject_t &hoid,   ///< [in] Object containing omap
-    bufferlist *header       ///< [out] omap header
+    bufferlist *header,      ///< [out] omap header
+    bool allow_eio = false ///< [in] don't assert on eio
     ) = 0;
 
   /// Get keys defined on hoid
@@ -768,6 +952,10 @@ public:
     
   virtual void set_fsid(uuid_d u) = 0;
   virtual uuid_d get_fsid() = 0;
+
+  // DEBUG
+  virtual void inject_data_error(const hobject_t &oid) {}
+  virtual void inject_mdata_error(const hobject_t &oid) {}
 };
 
 
