@@ -34,6 +34,7 @@
 #include "include/buffer.h"
 #include "include/xlist.h"
 #include "include/atomic.h"
+#include "SnapMapper.h"
 
 #include "OpRequest.h"
 #include "OSDMap.h"
@@ -41,6 +42,7 @@
 #include "msg/Messenger.h"
 #include "messages/MOSDRepScrub.h"
 #include "messages/MOSDPGLog.h"
+#include "common/tracked_int_ptr.hpp"
 
 #include <list>
 #include <memory>
@@ -63,6 +65,18 @@ class MOSDPGScan;
 class MOSDPGBackfill;
 class MOSDPGInfo;
 
+class PG;
+
+void intrusive_ptr_add_ref(PG *pg);
+void intrusive_ptr_release(PG *pg);
+
+#ifdef PG_DEBUG_REFS
+  uint64_t get_with_id(PG *pg);
+  void put_with_id(PG *pg, uint64_t id);
+  typedef TrackedIntPtr<PG> PGRef;
+#else
+  typedef boost::intrusive_ptr<PG> PGRef;
+#endif
 
 struct PGRecoveryStats {
   struct per_state_info {
@@ -85,7 +99,7 @@ struct PGRecoveryStats {
   }
   void dump(ostream& out) {
     Mutex::Locker l(lock);
-    for (map<const char *,per_state_info>::iterator p = info.begin(); p != info.end(); p++) {
+    for (map<const char *,per_state_info>::iterator p = info.begin(); p != info.end(); ++p) {
       per_state_info& i = p->second;
       out << i.enter << "\t" << i.exit << "\t"
 	  << i.events << "\t" << i.event_time << "\t"
@@ -115,7 +129,7 @@ struct PGRecoveryStats {
 };
 
 struct PGPool {
-  int id;
+  int64_t id;
   string name;
   uint64_t auid;
 
@@ -125,7 +139,7 @@ struct PGPool {
   interval_set<snapid_t> cached_removed_snaps;      // current removed_snaps set
   interval_set<snapid_t> newly_removed_snaps;  // newly removed in the last epoch
 
-  PGPool(int i, const char *_name, uint64_t au) :
+  PGPool(int64_t i, const char *_name, uint64_t au) :
     id(i), auid(au) {
     if (_name)
       name = _name;
@@ -211,7 +225,7 @@ public:
       caller_ops.clear();
       for (list<pg_log_entry_t>::iterator i = log.begin();
            i != log.end();
-           i++) {
+           ++i) {
         objects[i->soid] = &(*i);
 	if (i->reqid_is_indexed()) {
 	  //assert(caller_ops.count(i->reqid) == 0);  // divergent merge_log indexes new before unindexing old
@@ -360,6 +374,13 @@ public:
   /*** PG ****/
 protected:
   OSDService *osd;
+  OSDriver osdriver;
+  SnapMapper snap_mapper;
+public:
+  void update_snap_mapper_bits(uint32_t bits) {
+    snap_mapper.update_bits(bits);
+  }
+protected:
   OSDMapRef osdmap_ref;
   PGPool pool;
 
@@ -380,8 +401,15 @@ protected:
   Cond _cond;
   atomic_t ref;
 
+#ifdef PG_DEBUG_REFS
+  Mutex _ref_id_lock;
+  map<uint64_t, string> _live_ids;
+  map<string, uint64_t> _tag_counts;
+  uint64_t _ref_id;
+#endif
+
 public:
-  bool deleting;  // true while RemoveWQ should be chewing on us
+  bool deleting;  // true while in removing or OSD is shutting down
 
   void lock(bool no_lockdep = false);
   void unlock() {
@@ -414,17 +442,13 @@ public:
     _cond.Signal();
   }
 
-  void get() {
-    //generic_dout(0) << this << " " << info.pgid << " get " << ref.test() << dendl;
-    //assert(_lock.is_locked());
-    ref.inc();
-  }
-  void put() { 
-    //generic_dout(0) << this << " " << info.pgid << " put " << ref.test() << dendl;
-    if (ref.dec() == 0)
-      delete this;
-  }
-
+#ifdef PG_DEBUG_REFS
+  uint64_t get_with_id();
+  void put_with_id(uint64_t);
+  void dump_live_ids();
+#endif
+  void get(const string &tag);
+  void put(const string &tag);
 
   bool dirty_info, dirty_big_info, dirty_log;
 
@@ -432,6 +456,14 @@ public:
   // pg state
   pg_info_t        info;
   __u8 info_struct_v;
+  static const __u8 cur_struct_v = 7;
+  bool must_upgrade() {
+    return info_struct_v < 7;
+  }
+  void upgrade(
+    ObjectStore *store,
+    const interval_set<snapid_t> &snapcolls);
+
   const coll_t coll;
   IndexedLog  log;
   static string get_info_key(pg_t pgid) {
@@ -450,7 +482,7 @@ public:
   map<hobject_t, set<int> > missing_loc;
   set<int> missing_loc_sources;           // superset of missing_loc locations
   
-  interval_set<snapid_t> snap_collections;
+  interval_set<snapid_t> snap_collections; // obsolete
   map<epoch_t,pg_interval_t> past_intervals;
 
   interval_set<snapid_t> snap_trimq;
@@ -684,16 +716,19 @@ protected:
   void requeue_object_waiters(map<hobject_t, list<OpRequestRef> >& m);
   void requeue_ops(list<OpRequestRef> &l);
 
-  // stats
-  Mutex pg_stats_lock;
-  bool pg_stats_valid;
-  pg_stat_t pg_stats_stable;
+  // stats that persist lazily
+  object_stat_collection_t unstable_stats;
+
+  // publish stats
+  Mutex pg_stats_publish_lock;
+  bool pg_stats_publish_valid;
+  pg_stat_t pg_stats_publish;
 
   // for ordering writes
   std::tr1::shared_ptr<ObjectStore::Sequencer> osr;
 
-  void update_stats();
-  void clear_stats();
+  void publish_stats_to_osd();
+  void clear_publish_stats();
 
 public:
   void clear_primary_state();
@@ -750,7 +785,7 @@ public:
   void proc_master_log(ObjectStore::Transaction& t, pg_info_t &oinfo, pg_log_t &olog,
 		       pg_missing_t& omissing, int from);
   bool proc_replica_info(int from, const pg_info_t &info);
-  void remove_object_with_snap_hardlinks(
+  void remove_snap_mapped_object(
     ObjectStore::Transaction& t, const hobject_t& soid);
   bool merge_old_entry(ObjectStore::Transaction& t, pg_log_entry_t& oe);
 
@@ -832,7 +867,8 @@ public:
       reserved(false), reserve_failed(false),
       epoch_start(0),
       block_writes(false), active(false), queue_snap_trim(false),
-      waiting_on(0), errors(0), fixed(0), active_rep_scrub(0),
+      waiting_on(0), shallow_errors(0), deep_errors(0), fixed(0),
+      active_rep_scrub(0),
       must_scrub(false), must_deep_scrub(false), must_repair(false),
       classic(false),
       finalizing(false), is_chunky(false), state(INACTIVE),
@@ -851,7 +887,8 @@ public:
     bool queue_snap_trim;
     int waiting_on;
     set<int> waiting_on_whom;
-    int errors;
+    int shallow_errors;
+    int deep_errors;
     int fixed;
     ScrubMap primary_scrubmap;
     map<int,ScrubMap> received_maps;
@@ -861,7 +898,7 @@ public:
     // flags to indicate explicitly requested scrubs (by admin)
     bool must_scrub, must_deep_scrub, must_repair;
 
-    // Maps from objects with erros to missing/inconsistent peers
+    // Maps from objects with errors to missing/inconsistent peers
     map<hobject_t, set<int> > missing;
     map<hobject_t, set<int> > inconsistent;
     map<hobject_t, set<int> > inconsistent_snapcolls;
@@ -963,7 +1000,8 @@ public:
       start = hobject_t();
       end = hobject_t();
       subset_last_update = eversion_t();
-      errors = 0;
+      shallow_errors = 0;
+      deep_errors = 0;
       fixed = 0;
       deep = false;
       run_callbacks();
@@ -982,7 +1020,13 @@ public:
   map<int, ScrubMap *>::const_iterator _select_auth_object(
     const hobject_t &obj,
     const map<int,ScrubMap*> &maps);
-  bool _compare_scrub_objects(ScrubMap::object &auth,
+
+  enum error_type {
+    CLEAN,
+    DEEP_ERROR,
+    SHALLOW_ERROR
+  };
+  enum error_type _compare_scrub_objects(ScrubMap::object &auth,
 			      ScrubMap::object &candidate,
 			      ostream &errorstream);
   void _compare_scrubmaps(const map<int,ScrubMap*> &maps,  
@@ -1001,6 +1045,7 @@ public:
   void scrub_clear_state();
   bool scrub_gather_replica_maps();
   void _scan_list(ScrubMap &map, vector<hobject_t> &ls, bool deep);
+  void _scan_snaps(ScrubMap &map);
   void _request_scrub_map_classic(int replica, eversion_t version);
   void _request_scrub_map(int replica, eversion_t version,
                           hobject_t start, hobject_t end, bool deep);
@@ -1015,17 +1060,9 @@ public:
   virtual bool have_temp_coll() = 0;
   virtual bool _report_snap_collection_errors(
     const hobject_t &hoid,
+    const map<string, bufferptr> &attrs,
     int osd,
-    const map<string, bufferptr> &attrs,
-    const set<snapid_t> &snapcolls,
-    uint32_t nlinks,
     ostream &out) { return false; };
-  virtual void check_snap_collections(
-    ino_t hino, const hobject_t &hoid,
-    const map<string, bufferptr> &attrs,
-    set<snapid_t> *snapcolls) {};
-  void check_ondisk_snap_colls(
-    const interval_set<snapid_t> &ondisk_snapcolls);
   void clear_scrub_reserved();
   void scrub_reserve_replicas();
   void scrub_unreserve_replicas();
@@ -1041,9 +1078,29 @@ public:
   void sub_op_scrub_unreserve(OpRequestRef op);
   void sub_op_scrub_stop(OpRequestRef op);
 
+  void reject_reservation();
+  void schedule_backfill_full_retry();
 
   // -- recovery state --
 
+  template <class EVT>
+  struct QueuePeeringEvt : Context {
+    PGRef pg;
+    epoch_t epoch;
+    EVT evt;
+    QueuePeeringEvt(PG *pg, epoch_t epoch, EVT evt) :
+      pg(pg), epoch(epoch), evt(evt) {}
+    void finish(int r) {
+      pg->lock();
+      pg->queue_peering_event(PG::CephPeeringEvtRef(
+				new PG::CephPeeringEvt(
+				  epoch,
+				  epoch,
+				  evt)));
+      pg->unlock();
+    }
+  };
+
   class CephPeeringEvt {
     epoch_t epoch_sent;
     epoch_t epoch_requested;
@@ -1168,6 +1225,7 @@ public:
   TrivialEvent(RequestBackfill)
   TrivialEvent(RequestRecovery)
   TrivialEvent(RecoveryDone)
+  TrivialEvent(BackfillTooFull)
 
   TrivialEvent(AllReplicasRecovered)
   TrivialEvent(DoRecovery)
@@ -1363,8 +1421,10 @@ public:
       typedef boost::mpl::list <
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::custom_reaction< MNotifyRec >,
-	boost::statechart::transition< NeedActingChange, WaitActingChange >
+	boost::statechart::transition< NeedActingChange, WaitActingChange >,
+	boost::statechart::custom_reaction< AdvMap>
 	> reactions;
+      boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const ActMap&);
       boost::statechart::result react(const MNotifyRec&);
     };
@@ -1460,9 +1520,11 @@ public:
 
     struct Backfilling : boost::statechart::state< Backfilling, Active >, NamedState {
       typedef boost::mpl::list<
-	boost::statechart::transition< Backfilled, Recovered >
+	boost::statechart::transition< Backfilled, Recovered >,
+	boost::statechart::custom_reaction< RemoteReservationRejected >
 	> reactions;
       Backfilling(my_context ctx);
+      boost::statechart::result react(const RemoteReservationRejected& evt);
       void exit();
     };
 
@@ -1516,9 +1578,11 @@ public:
 
     struct RepRecovering : boost::statechart::state< RepRecovering, ReplicaActive >, NamedState {
       typedef boost::mpl::list<
-	boost::statechart::transition< RecoveryDone, RepNotRecovering >
+	boost::statechart::transition< RecoveryDone, RepNotRecovering >,
+	boost::statechart::custom_reaction< BackfillTooFull >
 	> reactions;
       RepRecovering(my_context ctx);
+      boost::statechart::result react(const BackfillTooFull &evt);
       void exit();
     };
 
@@ -1793,8 +1857,22 @@ public:
 private:
   void write_info(ObjectStore::Transaction& t);
   void write_log(ObjectStore::Transaction& t);
-public:
 
+public:
+  static void clear_info_log(
+    pg_t pgid,
+    const hobject_t &infos_oid,
+    const hobject_t &log_oid,
+    ObjectStore::Transaction *t);
+
+  static int _write_info(ObjectStore::Transaction& t, epoch_t epoch,
+    pg_info_t &info, coll_t coll,
+    map<epoch_t,pg_interval_t> &past_intervals,
+    interval_set<snapid_t> &snap_collections,
+    hobject_t &infos_oid,
+    __u8 info_struct_v, bool dirty_big_info, bool force_ver = false);
+  static void _write_log(ObjectStore::Transaction& t, pg_log_t &log,
+    const hobject_t &log_oid, map<eversion_t, hobject_t> &divergent_priors);
   void write_if_dirty(ObjectStore::Transaction& t);
 
   void add_log_entry(pg_log_entry_t& e, bufferlist& log_bl);
@@ -1821,11 +1899,11 @@ public:
   void read_state(ObjectStore *store, bufferlist &bl);
   static epoch_t peek_map_epoch(ObjectStore *store, coll_t coll,
                                hobject_t &infos_oid, bufferlist *bl);
-  coll_t make_snap_collection(ObjectStore::Transaction& t, snapid_t sn);
-  void update_snap_collections(vector<pg_log_entry_t> &log_entries,
-			       ObjectStore::Transaction& t);
+  void update_snap_map(
+    vector<pg_log_entry_t> &log_entries,
+    ObjectStore::Transaction& t);
+
   void filter_snapc(SnapContext& snapc);
-  void adjust_local_snaps();
 
   void log_weirdness();
 
@@ -1900,7 +1978,7 @@ public:
   void handle_loaded(RecoveryCtx *rctx);
   void handle_query_state(Formatter *f);
 
-  virtual void on_removal() = 0;
+  virtual void on_removal(ObjectStore::Transaction *t) = 0;
 
 
   // abstract bits
@@ -1931,9 +2009,4 @@ WRITE_CLASS_ENCODER(PG::OndiskLog)
 
 ostream& operator<<(ostream& out, const PG& pg);
 
-void intrusive_ptr_add_ref(PG *pg);
-void intrusive_ptr_release(PG *pg);
-
-typedef boost::intrusive_ptr<PG> PGRef;
-
 #endif
