@@ -28,13 +28,13 @@ typename T::iterator rand_choose(T &cont) {
   int index = rand() % cont.size();
   typename T::iterator retval = cont.begin();
 
-  for (; index > 0; --index) retval++;
+  for (; index > 0; --index) ++retval;
   return retval;
 }
 
 string num_str(unsigned i) {
   char buf[100];
-  snprintf(buf, sizeof(buf), "%.10d", i);
+  snprintf(buf, sizeof(buf), "%.10u", i);
   return string(buf);
 }
 
@@ -519,7 +519,7 @@ public:
 
     cerr << "using path " << strpath << std::endl;;
     LevelDBStore *store = new LevelDBStore(strpath);
-    assert(!store->init(cerr));
+    assert(!store->create_and_open(cerr));
 
     db.reset(new DBObjectMap(store));
     tester.db = db.get();
